package ch.netzschmiede.flarmradar.srv.flarmdata;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.slf4j.Logger;

import ch.netzschmiede.flarmradar.api.flarmdata.IFlarmDataMgrRemote;
import ch.netzschmiede.flarmradar.api.flarmdata.domain.FlarmRecord;
import ch.netzschmiede.flarmradar.api.flarmdata.util.QueryResult;
import ch.netzschmiede.flarmradar.lib.log.FlarmLoggerFactory;
import ch.netzschmiede.flarmradar.srv.flarmdata.entity.FlarmRecordEntity;

@Stateless
@LocalBean
public class FlarmDataMgr implements IFlarmDataMgrRemote {
	
	private static Logger log = FlarmLoggerFactory.getLoggerFor(FlarmDataMgr.class);
	
	private static final Pattern gpgga = Pattern.compile("^\\$GPGGA");
	private static final Pattern pflaa = Pattern.compile("^\\$PFLAA");

	@PersistenceContext(unitName = "flarmdata")
	EntityManager em;
	
	@Override
	public void write(Date date, String record) {
		if (null != record) {
			
			log.info("Store record: " + record);
			FlarmRecordEntity r = new FlarmRecordEntity();
			r.setDate(date);
			r.setRecord(record);
			em.persist(r);
			
			if (gpgga.matcher(record).find()) {
				// Meaning: Global Positioning System Fix Data  (see http://aprs.gids.nl/nmea/#gga)
				// this is our own position
				
			} else if (pflaa.matcher(record).find()) {
				// Meaning: Data on other aircraft around
				
			} else {
				// we ignore all other records
				log.debug("Ignore unused record: " + record);
			}
		}
	}

	
	@Override
	public void write(Date date, String[] records) {
		for (String record : records) {
			write(date, record);
		}
	}


	@Override
	public QueryResult read(Date date) {
		return read(date, 0, IFlarmDataMgrRemote.MAX_RESULT_COUNT);
	}


	@Override
	public QueryResult read(Date date, int offset) {
		return read(date, offset, IFlarmDataMgrRemote.MAX_RESULT_COUNT);
	}


	@Override
	public QueryResult read(Date date, int offset, int resultCount) {
		QueryResult qr = new QueryResult();
		
		Query count = em.createQuery("SELECT COUNT(r) FROM FlarmRecordEntity r WHERE r.date = :date");
		count.setParameter("date", date);
		int rowCount = ((Long)count.getSingleResult()).intValue();
		qr.setTotalCount(rowCount);
		if (rowCount > offset + IFlarmDataMgrRemote.MAX_RESULT_COUNT) {
			qr.setHasMore(true);
			qr.setNextId(offset + IFlarmDataMgrRemote.MAX_RESULT_COUNT);			
		}
		
		Query query = em.createQuery("SELECT r FROM FlarmRecordEntity r WHERE r.date = :date");
		query.setParameter("date", date);
		
		// fetch the records
		query.setFirstResult(offset);
		query.setMaxResults(IFlarmDataMgrRemote.MAX_RESULT_COUNT);
		
		@SuppressWarnings("unchecked")
		List<FlarmRecordEntity> result = query.getResultList();
		
		ArrayList<FlarmRecord> resultList = new ArrayList<FlarmRecord>();
		Iterator<FlarmRecordEntity> i = result.iterator();
		while (i.hasNext()) {
			resultList.add(i.next().toDomainObject());
		}
		qr.setRecord(resultList.toArray(new FlarmRecord[resultList.size()]));
		return qr;
	}

}
